/* c206.c **********************************************************}
{* Tma: Dvousmrn vzan linern seznam
**
**                   Nvrh a referenn implementace: Bohuslav Kena, jen 2001
**                            Pepracovan do jazyka C: Martin Tuek, jen 2004
**                                            pravy: Bohuslav Kena, jen 2010
**
** Implementujte abstraktn datov typ dvousmrn vzan linern seznam.
** Uitenm obsahem prvku seznamu je hodnota typu int.
** Seznam bude jako datov abstrakce reprezentovn promnnou
** typu tDLList (DL znamen Double-Linked a slou pro odlien
** jmen konstant, typ a funkc od jmen u jednosmrn vzanho linernho
** seznamu). Definici konstant a typ naleznete v hlavikovm souboru c206.h.
**
** Vam kolem je implementovat nsledujc operace, kter spolu
** s ve uvedenou datovou st abstrakce tvo abstraktn datov typ
** obousmrn vzan linern seznam:
**
**      DLInitList ...... inicializace seznamu ped prvnm pouitm,
**      DLDisposeList ... zruen vech prvk seznamu,
**      DLInsertFirst ... vloen prvku na zatek seznamu,
**      DLInsertLast .... vloen prvku na konec seznamu, 
**      DLFirst ......... nastaven aktivity na prvn prvek,
**      DLLast .......... nastaven aktivity na posledn prvek, 
**      DLCopyFirst ..... vrac hodnotu prvnho prvku,
**      DLCopyLast ...... vrac hodnotu poslednho prvku, 
**      DLDeleteFirst ... zru prvn prvek seznamu,
**      DLDeleteLast .... zru posledn prvek seznamu, 
**      DLPostDelete .... ru prvek za aktivnm prvkem,
**      DLPreDelete ..... ru prvek ped aktivnm prvkem, 
**      DLPostInsert .... vlo nov prvek za aktivn prvek seznamu,
**      DLPreInsert ..... vlo nov prvek ped aktivn prvek seznamu,
**      DLCopy .......... vrac hodnotu aktivnho prvku,
**      DLActualize ..... pepe obsah aktivnho prvku novou hodnotou,
**      DLSucc .......... posune aktivitu na dal prvek seznamu,
**      DLPred .......... posune aktivitu na pedchoz prvek seznamu, 
**      DLActive ........ zjiuje aktivitu seznamu.
**
** Pi implementaci jednotlivch funkc nevolejte dnou z funkc
** implementovanch v rmci tohoto pkladu, nen-li u funkce
** explicitn uvedeno nco jinho.
**
** Nemuste oetovat situaci, kdy msto leglnho ukazatele na seznam 
** ped nkdo jako parametr hodnotu NULL.
**
** Svou implementaci vhodn komentujte!
**
** Terminologick poznmka: Jazyk C nepouv pojem procedura.
** Proto zde pouvme pojem funkce i pro operace, kter by byly
** v algoritmickm jazyce Pascalovskho typu implemenovny jako
** procedury (v jazyce C procedurm odpovdaj funkce vracejc typ void).
**/

#include "c206.h"

int solved;
int errflg;

void DLError() {
/*
** Vytiskne upozornn na to, e dolo k chyb.
** Tato funkce bude volna z nkterch dle implementovanch operac.
**/	
    printf ("*ERROR* Chyba pi prci se seznamem.\n");
    errflg = TRUE;             /* globln promnn -- pznak oeten chyby */
    return;
}

void DLInitList (tDLList *L)	{
/*
** Provede inicializaci seznamu L ped jeho prvnm pouitm (tzn. dn
** z nsledujcch funkc nebude volna nad neinicializovanm seznamem).
** Tato inicializace se nikdy nebude provdt nad ji inicializovanm
** seznamem, a proto tuto monost neoetujte. Vdy pedpokldejte,
** e neinicializovan promnn maj nedefinovanou hodnotu.
**/
	L->First = NULL;
	L->Act = NULL;
	L->Last = NULL;
}

void DLDisposeList (tDLList *L)	{
/*
** Zru vechny prvky seznamu L a uvede seznam do stavu, v jakm
** se nachzel po inicializaci. Ruen prvky seznamu budou korektn
** uvolnny volnm operace free. 
**/

	tDLElemPtr pom;

	while ( (pom = L->First) != NULL )
	{
		L->First = L->First->rptr;  //prenastaveni prvniho prvku
		free(pom);
	}

	L->Last = NULL;
	L->Act = NULL;

}

void DLInsertFirst (tDLList *L, int val)	{
/*
** Vlo nov prvek na zatek seznamu L.
** V ppad, e nen dostatek pamti pro nov prvek pi operaci malloc,
** vol funkci DLError().
**/

	tDLElemPtr newItem;

	newItem = malloc(sizeof(struct tDLElem));

	if ( newItem == NULL )
		DLError();
	else
	{
		newItem->data = val; //zapis hodnoty prvku
		newItem->rptr = L->First; //posunuti praveho ukazatele
		newItem->lptr = NULL;	//prvni prvek levy ukazatel nema

		if ( L->First == NULL ) //pokud jde o prvni vkladany prvek
			L->Last = newItem;
		else
			//levy ukazatel ted uz druheho prvku musi ukazovat na prvni:
			L->First->lptr = newItem;

		L->First = newItem;//ukazatel na prvni prvek seznamu
		
	}
}

void DLInsertLast(tDLList *L, int val)	{
/*
** Vlo nov prvek na konec seznamu L (symetrick operace k DLInsertFirst).
** V ppad, e nen dostatek pamti pro nov prvek pi operaci malloc,
** vol funkci DLError().
**/

	tDLElemPtr newItem;

	newItem = malloc(sizeof(struct tDLElem));

	if ( newItem == NULL )
		DLError();
	else
	{
		newItem->data = val;
		newItem->rptr = NULL; //nema praveho ukazatele
		newItem->lptr = L->Last; // levy ukazatel ukazuje na predposledni prvek

		if ( L->First == NULL ) //pokud jde o prvni vkladany prvek
			L->First = newItem;
		else
			//levy ukazatel ted uz druheho prvku musi ukazovat na prvni:
			L->Last->rptr = newItem;

		L->Last = newItem; //ukazatel na posledni prvek seznamu
	}

}

void DLFirst (tDLList *L)	{
/*
** Nastav aktivitu na prvn prvek seznamu L.
** Funkci implementujte jako jedin pkaz (nepotme-li return),
** ani byste testovali, zda je seznam L przdn.
**/

	L->Act = L->First;
}

void DLLast (tDLList *L)	{
/*
** Nastav aktivitu na posledn prvek seznamu L.
** Funkci implementujte jako jedin pkaz (nepotme-li return),
** ani byste testovali, zda je seznam L przdn.
**/
	L->Act = L->Last;
}

void DLCopyFirst (tDLList *L, int *val)	{
/*
** Vrt hodnotu prvnho prvku seznamu L.
** Pokud je seznam L przdn, vol funkci DLError().
**/
	if ( L->First == NULL )//podminka na prazdny seznam
		DLError();
	else 
		*val = L->First->data;
}

void DLCopyLast (tDLList *L, int *val)	{
/*
** Vrt hodnotu poslednho prvku seznamu L.
** Pokud je seznam L przdn, vol funkci DLError().
**/
	if ( L->First == NULL )//podminka na prazdny seznam
		DLError();
	else 
		*val = L->Last->data;
}

void DLDeleteFirst (tDLList *L)	{
/*
** Zru prvn prvek seznamu L. Pokud byl prvn prvek aktivn, aktivita 
** se ztrc. Pokud byl seznam L przdn, nic se nedje.
**/
	if ( L->First != NULL )
	{//seznam neni prazdny

		tDLElemPtr pom;
		pom = L->First;

		if(L->Act == L->First)
			L->Act = NULL;  //odstraneni aktivity

		if(L->First != L->Last)
		{//pokud je v seznamu vice nez jeden prvek
			L->First = L->First->rptr;//prvni prvek bude co byl druhy
			L->First->lptr = NULL;//levy ukazatel neukazuje nikam
		}
		else
		{// pokud je v seznamu jen jeden prvek
			L->First = NULL;
			L->Last = NULL;
		}
		free(pom);
	}

}	

void DLDeleteLast (tDLList *L)	{
/*
** Zru posledn prvek seznamu L. Pokud byl posledn prvek aktivn,
** aktivita seznamu se ztrc. Pokud byl seznam L przdn, nic se nedje.
**/
	if ( L->First != NULL )
	{//seznam neni prazdny

		tDLElemPtr pom;
		pom = L->Last;

		if(L->Act == L->Last)
			L->Act = NULL;  //odstraneni aktivity

		if(L->First != L->Last)
		{//pokud je v seznamu vice nez jeden prvek
			L->Last = L->Last->lptr;//prvni prvek bude co byl druhy
			L->Last->rptr = NULL;//levy ukazatel neukazuje nikam
		}
		else
		{// pokud je v seznamu jen jeden prvek
			L->First = NULL;
			L->Last = NULL;
		}
		free(pom);
	}
}

void DLPostDelete (tDLList *L)	{
/*
** Zru prvek seznamu L za aktivnm prvkem.
** Pokud je seznam L neaktivn nebo pokud je aktivn prvek
** poslednm prvkem seznamu, nic se nedje.
**/
	// overeni aktivity seznamu a overeni prvniho prvku:
	if((L->Act != NULL) && (L->First != NULL) && (L->Act != L->Last))
	{
		tDLElemPtr pom;
		pom = L->Act->rptr;  //pomocny ukazatel

		L->Act->rptr = pom->rptr;	//preskoceni prvku z prave strany

		if(pom == L->Last)
			L->Last = L->Act;//nastaveni noveho prvniho prvku, pokud byl smazan
		else
			pom->rptr->lptr = L->Act;	//preskoceni prvku z leve strany
		free(pom);
	}
}

void DLPreDelete (tDLList *L)	{
/*
** Zru prvek ped aktivnm prvkem seznamu L .
** Pokud je seznam L neaktivn nebo pokud je aktivn prvek
** prvnm prvkem seznamu, nic se nedje.
**/

	// overeni aktivity seznamu a overeni prvniho prvku:
	if((L->Act != NULL) && (L->First != NULL) && (L->Act != L->First))
	{
		tDLElemPtr pom;
		pom = L->Act->lptr;

		L->Act->lptr = pom->lptr;	//preskoceni prvku z prave strany

		if(pom == L->First)
			L->First = L->Act;//nastaveni noveho prvniho prvku, pokud byl smazan
		else
			pom->lptr->rptr = L->Act;  //preskoceni prvku z leve strany
		free(pom);
  }
}

void DLPostInsert (tDLList *L, int val) {
/*
** Vlo prvek za aktivn prvek seznamu L.
** Pokud nebyl seznam L aktivn, nic se nedje.
** V ppad, e nen dostatek pamti pro nov prvek pi operaci malloc,
** vol funkci DLError().
**/
	if(L->Act != NULL)	//overeni aktivity seznamu
	{
		tDLElemPtr newItem;

		newItem = malloc(sizeof(struct tDLElem));

		if (newItem == NULL)
			DLError();
		else
		{
			newItem->data = val;
			newItem->lptr = L->Act;
			newItem->rptr = L->Act->rptr;
			//navazani noveho prvku do seznamu z leve strany
			L->Act->rptr = newItem;
			if(L->Act == L->Last)
				L->Last = newItem; //nastaveni noveho posledniho prvku
			else
				//navazani noveho prvku do seznamu z prave strany
				newItem->rptr->lptr = newItem;
		}
	}
}

void DLPreInsert (tDLList *L, int val)		{
/*
** Vlo prvek ped aktivn prvek seznamu L.
** Pokud nebyl seznam L aktivn, nic se nedje.
** V ppad, e nen dostatek pamti pro nov prvek pi operaci malloc,
** vol funkci DLError().
**/
	if(L->Act != NULL)
	{

		tDLElemPtr newItem = malloc(sizeof(struct tDLElem));

		if (newItem == NULL)
			DLError();
		else
		{
			newItem->data = val;
			newItem->lptr = L->Act->lptr;
			newItem->rptr = L->Act;
			if(L->Act == L->First)
				L->First = newItem;  //zapis noveho prvniho prvku
			else
				//navazani noveho prvku z leve strany:
				newItem->lptr->rptr = newItem;

			L->Act->lptr = newItem;//navazani noveho prvku z prave strany:
		}
	}
}

void DLCopy (tDLList *L, int *val)	{
/*
** Vrt hodnotu aktivnho prvku seznamu L.
** Pokud seznam L nen aktivn, vol funkci DLError ().
**/
	if(L->Act == NULL)
		DLError();
	else
		*val = L->Act->data;
}

void DLActualize (tDLList *L, int val) {
/*
** Pepe obsah aktivnho prvku seznamu L.
** Pokud seznam L nen aktivn, nedl nic.
**/
	if(L->Act != NULL)
		L->Act->data = val;
}

void DLSucc (tDLList *L)	{
/*
** Posune aktivitu na nsledujc prvek seznamu L.
** Nen-li seznam aktivn, nedl nic.
** Vimnte si, e pi aktivit na poslednm prvku se seznam stane neaktivnm.
**/
	if(L->Act != NULL)  //overeni aktivity seznamu
	{
		if(L->Act != L->Last)
			L->Act = L->Act->rptr;	//posunuti aktivity na dalsi prvek
		else
			L->Act = NULL;	//posledni prvek nema ukazatel
	}
}


void DLPred (tDLList *L)	{
/*
** Posune aktivitu na pedchoz prvek seznamu L.
** Nen-li seznam aktivn, nedl nic.
** Vimnte si, e pi aktivit na prvnm prvku se seznam stane neaktivnm.
**/
	if(L->Act != NULL)  //overeni aktivity seznamu
	{
		if(L->Act != L->First)
			L->Act = L->Act->lptr;  // posunuti aktivity na predchozi prvek
		else
			L->Act = NULL;  //prvni prvek nema ukazatel
	}
}

int DLActive (tDLList *L) {		
/*
** Je-li seznam aktivn, vrac true. V opanm ppad vrac false.
** Funkci implementujte jako jedin pkaz.
**/
	return (L->Act != NULL);
}

/* Konec c206.c*/
